import com.gizmet.util.*;
import com.gizmet.tenochca.map.*;
import com.gizmet.tenochca.model.*;
import com.gizmet.ui.*;

class com.gizmet.tenochca.Game extends MovieClip
{
	// This is the master class for the Tenochca game.
	private var screenNames:Array = ["intro","map"];
	private var screens:Array;
	
	private var broadcaster:EventBroadcaster;
	private var keyHandler:KeyHandler;
	private var map:Map;
	private var model:Model;
	private var inited:Boolean = false;
	
	public function Game () 
	{
		// general setup stuffs
		// TBD - still need a loader thingy
// 		Logger.doTrace = true;
		
		Stage.scaleMode = "noScale";
		SerialIterator.initialize(1000);
		Codex.initialize();
		this.broadcaster = EventBroadcaster.getBroadcaster();
		this.broadcaster.addEventListener("StartPressed", this);
		this.broadcaster.addEventListener("DisplayExposition", this);
		this.broadcaster.addEventListener("DisplayAlert", this);
		this.map = Map.getMap();
		this.model = Model.getModel();
		this.keyHandler = KeyHandler.getKeyHandler();
		this.keyHandler.setFocus("Game");
		
		var game_key_handler = function (target, keyCode) {
			if (keyCode == Key.ENTER or keyCode == Key.SPACE) {
				target.broadcaster.broadcast("StartPressed");
			}
		}
		
		this.keyHandler.setCallbacks("Game", this, game_key_handler);
		
		this.init();
		this.displayScreen("intro");
	}
	
	private function init ()
	{
		if (this.inited) {
			return;
		}
		this.model.init();
		
		// load all the screen movieclips onto the scene
		this.screens = new Array();
		for (var i in this.screenNames) {
			var screenName = "screen_"+this.screenNames[i];
			var screen = this.attachMovie(screenName, screenName, SerialIterator.getSerial());
			screen.id = this.screenNames[i];
			screen.hideScreen();
			this.screens.push(screen);
		}

		this.inited = true;
	}
	
	private function displayScreen(id)
	{
		for (var i in this.screens) {
			var screen = this.screens[i];
			if (screen.id eq id) {
				screen.displayScreen();
			}
			else {
				screen.hideScreen();
			}
		}
	}
	
	// Event Handlers
	
	private function StartPressed (obj) 
	{
		this.keyHandler.setFocus("ScreenMap");
		this.displayScreen("map");
		this.broadcaster.broadcast("DisplayExposition", {text: Codex.getExposition("intro")});
	}
	
	private function DisplayExposition (obj)
	{
		var expo = this.attachMovie("dialogExposition", "expo", SerialIterator.getSerial());
		expo.setText(obj.data.text);
		expo._x = 116;
		expo._y = 116;
	}
	
	private function DisplayAlert (obj)
	{
		var alert = this.attachMovie("dialogAlert", "alert", SerialIterator.getSerial());
		alert.setText(obj.data.text);
		alert._x = 120;
		alert._y = 200;
	}
	
}
