import com.gizmet.tenochca.map.*;
import com.gizmet.tenochca.model.Model;

/*
This class represents the data structure of the Map, not the visual
representation of it - that lives in com.gizmet.tenochca.ui.MapViewer.
*/

class com.gizmet.tenochca.map.Map 
{	
	private static var instanceCount:Number = 0;
	private static var myMap:Map;
	private var model:Model;

	public var grid:Array;
	public var occupiedSquares:Object;
	public var mapWidth = 34;
	public var mapHeight = 44;
	public var tileWidth = 64;
	public var tileHeight = 64;
	
	public var tileset:Object = {
		b:		"tile_blank",
		h:		"tile_hills",
		j:		"tile_impenetrableJungle",
		m:		"tile_mountain",
		o:		"tile_open",
		p:		"tile_port",
		r1:		"tile_river1",
		r2:		"tile_river2",
		r3:		"tile_river3",
		r4:		"tile_river4",
		r5:		"tile_river5",
		r6:		"tile_river6",
		r7:		"tile_river7",
		r8:		"tile_river8",
		r9:		"tile_river9",
		r10:	"tile_river10",
		r11:	"tile_river11",
		r12:	"tile_river12",
		r13:	"tile_river13",
		r14:	"tile_river14",
		r15:	"tile_river15",
		u:		"tile_unseen"
	};
	
	public static function getMap () 
	{
	// implement singleton pattern
		if (Map.instanceCount < 1) {
			Map.myMap = new Map();
		}
		return Map.myMap;
	}

	private function Map ()
	{
		Map.instanceCount++;
		this.model = Model.getModel();
		this.grid = new Array();
	}
	
	public function init()
	{
		this.occupiedSquares = new Object();
		// This initialization function should do everything - loading the structure from
		// strings, generating random terrain, filling all the goodies, and so on.
		TerrainLoader.initTerrain(this);
		// generate the river and hills
		TerrainGenerator.generateRiver(this);
		TerrainGenerator.generateHills(this);
		this.occupiedSquares = new Object();
	}
	
	public function setSquareType (x,y,myType)
	{
		// Sets the terrain type of the square at the given coordinates
		this.getSquare(x,y).type = myType;
		if (myType ne "o") {
			this.markOccupied(x,y);
		}
	}
	
	public function setRiverSquare (x,y,riverNum)
	{
		// Sets the river number of a given square, and sets the proper
		// square terrain type accordingly
		this.setSquareType(x,y,"r"+riverNum);
		this.getSquare(x,y).riverNum = riverNum;
		
	}
	
	public function getSquare (x,y)
	{
		// returns the mapSquare object for the given coordinates
		return this.grid[y][x];
	}
	
	public function getTile (x,y)
	{
		// returns the name of the tile movieclip for the given coordinates
		var square = this.getSquare(x,y);
		if (square == undefined) {
			return this.tileset["u"];
		}
		else {
			return this.tileset[square.type];
		}
	}
	
	public function markOccupied (x,y)
	{
		var key = x+"_"+y;
		this.occupiedSquares[key] = true;
	}
	
	public function isOccupied (x,y)
	{
		var key = x+"_"+y;
		if (this.occupiedSquares[key]) {
			return(true);
		}
		else {
			return(false);
		}
	}
	
	public function canMove(x,y) 
	{
		// checks the given map coordinates to see if the player can move there or not
		var square = this.getSquare(x,y);
		if (x < 0 or y < 0 or x > this.mapWidth or y > this.mapHeight) {
			return(false);
		}
		else if (square.type eq "m") {
			return(false);
		}
		return(true);
	}
	
	public function toString()
	{
		var typeToTile = {
			b: '.',	h: 'H', 	j: '<', 	m: '^',	o: '#',	p: 'O',	u:	' ',
			r1: '+',	r2: '+',	r3: '+',	r4: '+',	r5: '+',	r6: '+',	r7: '+',	r8: '+',
			r9: '+',	r10: '+',	r11: '+',	r12: '+',	r13: '+',	 r14: '+', 	r15: '+'
		}
		var gridText = '';
		for (var i=0; i<this.grid.length; i++) {
			for (var j=0; j<this.grid[i].length; j++) {
				gridText += typeToTile[grid[i][j].type];
			}
			gridText += "\n";
		}
		
		return(gridText);
	}
	
}