import com.gizmet.tenochca.map.*;
import com.gizmet.tenochca.model.Model;

/*
	This is a primarily static class that provides some handy terrain generation
	methods to make the game map all random and interesting.  Most of these 
	methods take a Map object as an argument, and so on.
*/

class com.gizmet.tenochca.map.TerrainGenerator 
{	
	private static var UP:Number = 8;
	private static var RIGHT:Number = 4;
	private static var DOWN:Number =2;
	private static var LEFT:Number = 1;
	private static var __dirs__ = [UP, RIGHT, DOWN, LEFT];
	
	public static function generateRiver (map)
	{
		var model = Model.getModel();
		var mouth_x = model.mouth_x;
		var mouth_y = model.mouth_y;
		
		// drop in initial river squares
		map.setRiverSquare(mouth_x,mouth_y, 8);
		map.setRiverSquare(mouth_x,mouth_y-1, 10);

		// add river sources

		var riverSources = new Array();
		for (var i=0; i<model.num_river_sources; i++) {
			var riverSourceFunc = function () {
				var rx = Math.floor(Math.random() * (map.mapWidth-(model.river_source_x_buffer * 2)) + model.river_source_x_buffer);
				var ry =model.river_source_max - Math.floor(Math.random() * model.river_source_y_range);
				return new Array(rx, ry, getRiverSource());
			}
			var sxy = addWithSpacer(map, model.river_source_spacer, riverSourceFunc);
			riverSources.push(sxy);
		}

		for (var i=0; i<model.num_river_sources; i++) {
			var riverSourceFunc = function () {
				var rx = Math.floor(Math.random() * (map.mapWidth- (model.river_source_x_buffer * 2)) + model.river_source_x_buffer);
				var ry = Math.floor(Math.random() * (model.river_source_max - model.river_source_y_range - (model.river_source_x_buffer * 4)) + (model.river_source_x_buffer*3));
				return new Array(rx, ry, getRiverSource());
			}
			var sxy = addWithSpacer(map, model.river_source_spacer, riverSourceFunc);
			riverSources.push(sxy);
		}

		// trace the river from the sources to the mouth

		for (var i=0; i<riverSources.length; i++) {
			var myX = riverSources[i][0];
			var myY = riverSources[i][1];
			var lastX = -1;
			var lastY = -1;
			var count = 0;
			while (myX != model.mouth_x or myY != model.mouth_y) {		
				//	figure out which end is open
				var nextTile = getNextOpenTile(map, myX, myY, lastX, lastY);

				//	move that direction
				var nextX = nextTile[0];
				var nextY = nextTile[1];
				var cameFromDir = nextTile[2];
				
				// if you run into another river tile, make the connection right, and bail.
				var curTile = map.getSquare(nextX, nextY);
				if (curTile.riverNum > 0) {
					map.setRiverSquare(nextX, nextY, curTile.riverNum | cameFromDir);
					break;
				}
				
				//	figure out where the river mouth is in relation to new position
				var dX = model.mouth_x - nextX;
				var dY = model.mouth_y - nextY;
				
				//	select random exit point based on that vector, choose exit direction
				var exitDir = 0;
				var absX = Math.abs(dX) + model.fudge_wave;
				var absY = Math.abs(dY) + model.fudge_wave;
				var dTotal = absX  + absY;

				if (nextY < model.river_source_x_buffer or Math.random() < (absX * (1/dTotal))) {
					if (dX >= 0) {
						exitDir = RIGHT;
					}
					else {
						exitDir = LEFT;
					}
				}
				else {
					if (dY > 0)  {
						exitDir = DOWN; 
					}
					else {
						exitDir = UP;
					}
				}
				
				if (exitDir == cameFromDir) { 
					//	check tile in that direction for obstacle TBD
					//		if there's something in the way, turn right or left randomly
					exitDir = turnRandomDir(exitDir);
				}
				
				//	figure out which tile corresponds to entrance and exit points, place it
				var newTileValue = exitDir | cameFromDir;
				map.setRiverSquare(nextX,nextY,newTileValue);

				lastX = myX;
				lastY = myY;
				myX = nextX;
				myY = nextY;

				if (count++>model.max_map_tries) {
					break;
				}
			}		
		}

	}

	private static function getRiverSource ()
	{
		// returns a random "end" river tile
		var r = Math.random();
		if (r<.5) {
			return 8;
		}
		else if (r<.75) {
			return 1;
		}
		else{
			return 4;
		}
	}
	
	private static function getNextOpenTile (map, myX, myY, lastX, lastY) {
		// returns the coordinates of the next open tile from given one,
		// that is not the last x and y given
		var myTile = map.getSquare(myX, myY);
	
		var myDirs = new Array();
		for (var i in __dirs__) {
			if (myTile.riverNum & __dirs__[i]) {
				myDirs.push(__dirs__[i]);
			}
		}
	
		for (var i in myDirs) {
			var nTile = getNeighbor(map, myX, myY, myDirs[i]);
			if (nTile[0] != lastX or nTile[1] != lastY) {
				nTile.push(oppDir(myDirs[i]));
				return nTile;
			}
		}
	
		return false;
	}

	private static function getNeighbor (map, x, y, dir) {
		switch (dir) {
			case UP:
				if (y>0) {
					return new Array(x, y-1);
				}
				break;
			case RIGHT:
				if (x < (map.mapWidth-1)) {
					return new Array(x+1,y);
				}
				break;
			case DOWN:
				if (y < (map.mapHeight-1)) {
					return new Array(x, y+1);
				}
				break;
			case LEFT:
				if (x>0) {
					return new Array(x-1, y);
				}
				break;
		}
		return false;
	}
	

	// Run for the hills!
	
	public static function generateHills (map)
	{
		var model = Model.getModel();
		var hillfunc = function (model) {
			var hx = Math.floor(Math.random() * map.mapWidth);
			var by = 1-(Math.random() * Math.random());
			var hy = Math.floor(by * (model.river_source_max + model.river_source_x_buffer));
			return new Array(hx, hy, "h");
		}
		for (var i=0; i<model.num_hills; i++) {
			addWithSpacer(map, model.hill_spacer, hillfunc);	
		}
	}

	
	// Utility methods

	private static function addWithSpacer (map, spacer, xyvFunction) {
		// attempts to add a tile at the point generated by the given function and spacing,
		// repeating as many times as necessary, or until a set limit is reached.
		// returns an array containing the x and y coordinates set.
		var model = Model.getModel();
		var done = false;
		var tmpTries = 0;
		var ret = new Array();
	
		while (not done) {
			done = true;
			if (tmpTries++ > model.max_map_tries) {
				trace("Too many thingers to fit in map");
// 				break;
				spacer--;
				tmpTries = 0;
			}
			
			ret = xyvFunction(model);
			done = canAddWithSpacer(map, ret[0], ret[1], spacer);
		}
		
		if (Number(ret[2]) == NaN or ret[2] eq "h") {
			map.setSquareType(ret[0],ret[1],ret[2]);
		}
		else { // river number
			map.setRiverSquare(ret[0],ret[1],ret[2]);
		}
		return new Array(ret[0], ret[1]);
	}

	private static function canAddWithSpacer (map, x, y, spacer) {
		// returns true if the given point can be safely inserted with the given spacing
		for (var y2=y-spacer; y2<=y+spacer; y2++) {
			for (var x2=x-spacer; x2<=x+spacer; x2++) {
				if (absDistance(x,y,x2,y2) < spacer and map.isOccupied(x2,y2)) {
					return(false);
				}
			}
		}
		return(true);
	}

	private static function absDistance (x1, y1, x2, y2) {
		// computes "rectangular" distance between two points
		return Math.abs(x1-x2) + Math.abs(y1-y2);
	}
	
	private static function dirString (dir) {
		switch(dir) {
			case UP:
				return "UP";
			case DOWN:
				return "DOWN";
			case RIGHT:
				return "RIGHT";
			case LEFT:
				return "LEFT";
		}
		return dir;
	}
	
	private static function oppDir (dir) {
		switch(dir) {
			case UP:
				return DOWN;
			case DOWN:
				return UP;
			case RIGHT:
				return LEFT;
			case LEFT:
				return RIGHT;
		}
		return -1;
	}
	
	private static function turnRandomDir (dir) {
		switch(dir) {
			case UP:
			case DOWN:
				if (Math.random() < .5) {
					return RIGHT;
				}
				else {
					return LEFT;
				}
			case RIGHT:
			case LEFT:
				if (Math.random() < .5) {
					return UP;
				}
				else {
	// 				return DOWN;
					return UP;
				}
		}
		return -1;
	}

}
