import com.gizmet.util.*;
import com.gizmet.tenochca.map.*;
import com.gizmet.tenochca.model.*;

class com.gizmet.tenochca.ui.MapViewer extends MovieClip
{
	private var map:Map;
	private var model:Model;
	private var broadcaster:EventBroadcaster;
	private var keyHandler:KeyHandler;
	
	private var view_x:Number;
	private var view_y:Number;
	private var view_width:Number = 7;
	private var view_height:Number = 7;
	private var tile_width:Number ;
	private var tile_height:Number;
	private var tile_seed:Number;
	
	private var tileStub:MovieClip;
	private var tileClips:Array;
	private var extraClips:Array;
	private var cursor:MovieClip;
	
	public var revealMode:String;
	public static var revealDuration:Number = 500;
	public static var tileMoveDuration:Number = 300;
	
	public function MapViewer()
	{
		this.model = Model.getModel();
		this.map = Map.getMap();
		this.tile_width = map.tileWidth;
		this.tile_height = map.tileHeight;
		
		this.broadcaster = EventBroadcaster.getBroadcaster();
		this.broadcaster.addEventListener("MoveView", this);
		this.keyHandler = KeyHandler.getKeyHandler();
		this.keyHandler.setCallbacks("MapViewer", this, this.map_viewer_key_handler);

	}
	
	public function initMap()
	{
		Logger.log("MapViewer: initializing map");
		this.map.init();
		
		this.revealMode = "standard";
		this.view_x = this.model.port_x;
		this.view_y = this.model.port_y;
		this.tile_seed = Math.floor(Math.random()*20);
		this.extraClips = new Array();
		
		this.cursor = this.attachMovie("MapViewerCursor", "cursor", SerialIterator.getSerial());
		this.cursor._x = Math.floor(this.view_width/2) * this.tile_width;
		this.cursor._y = Math.floor(this.view_height/2) * this.tile_height;
		this.displayView();
	}
	
	public function displayView(x,y)
	{
		// Draws all the map tiles at the given coordinates, or at
		// the current view coordinates, if not given.
		if (x == undefined) {
			x = this.view_x;
		}
		if (y == undefined) {
			y = this.view_y;
		}
		Logger.log("MapViewer: displaying map at ("+x+","+y+")");
		this.removeViewClips();
		this.tileStub._x = 0;
		this.tileStub._y = 0;
		
		var left = x - Math.floor(this.view_width/2);
		var right = x + Math.floor(this.view_width/2);
		var top = y - Math.floor(this.view_height/2);
		var bottom = y + Math.floor(this.view_height/2);

		var x_count = 0;
		for (var display_x=left; display_x<=right; display_x++) {
			var clipColumn = new Array();
			var y_count = 0;
			for (var display_y=top; display_y<=bottom; display_y++) {
				var tileClip = this.placeTileClip(display_x, display_y, x_count * this.tile_width, y_count * this.tile_height);
				
				// show revealed squares where needed
				var reveal = false; 
				if (tileClip.mcUnseen._visible = true) {
					switch (this.revealMode) {
						case "standard":
							// reveal only current tile and directly adjacent ones
							if ((x_count == 3 and y_count <= 4 and y_count >=2) or (y_count == 3 and x_count <=4 and x_count >=2) ) {
								reveal = true;
							}
							break;
						case "all":
							reveal = true;
							break;
					}
				}
				if (reveal == true) {
					var square = this.map.getSquare(display_x, display_y);
					square.seen = true;
					tileClip.mcUnseen.revealTime = getTimer();
					tileClip.mcUnseen.onEnterFrame = function () {
						var u = (getTimer() - this.revealTime) / MapViewer.revealDuration;
						if (u >= 1) {
							this.removeMovieClip();
							this.onEnterFrame = undefined;
						}
						else {
							this._alpha = 100 - (100 * u);
						}
					}
				}
				
				clipColumn[y_count] = tileClip;
				y_count++;
			}
			this.tileClips[x_count] = clipColumn;
			x_count++;
		}
	}

	private function placeTileClip (display_x, display_y, view_x, view_y) 
	{
		var square = this.map.getSquare(display_x, display_y);
		var tile = this.map.getTile(display_x, display_y);
		var tileClip = this.tileStub.attachMovie(tile, "tile"+SerialIterator.getSerial(), SerialIterator.getSerial());
		var tileFrame = this.getFrameHash(display_x, display_y, tileClip._totalframes);
		tileClip.gotoAndStop(tileFrame);
		tileClip._x = view_x;
		tileClip._y = view_y;
		this.checkUnseen(square,tileClip);
		return(tileClip);
	}
	
	private function moveView (x_offset, y_offset) 
	{
		// slides the current view over, adding and removing rows or columns as necessary
		// first, add the new row/column // TBD - handle more than 1?
		if ((this.view_x+x_offset) < 0 or (this.view_y+y_offset) < 0 or (this.view_x+x_offset) >= this.map.mapWidth or (this.view_y+y_offset) > this.map.mapHeight) {
			return;
		}
		
		this.keyHandler.keyPressOkay = false;
		this.view_x += x_offset;
		this.view_y += y_offset;

		var left = view_x - Math.floor(this.view_width/2);
		var right = view_x + Math.floor(this.view_width/2);
		var top = view_y - Math.floor(this.view_height/2);
		var bottom = view_y + Math.floor(this.view_height/2);
		
		var build_row = false;
		var build_column = false;
		
		if (y_offset == 1) {
			var display_y = bottom;
			var x_count = 0;
			for (var display_x=left; display_x<=right; display_x++) {
				var tileClip = this.placeTileClip(display_x, display_y, x_count * this.tile_width, this.view_height * this.tile_height);
				this.extraClips.push(tileClip);
				x_count++;
			}
		}
		else if (y_offset == -1) {
			var display_y = top;
			var x_count = 0;
			for (var display_x=left; display_x<=right; display_x++) {
				var tileClip = this.placeTileClip(display_x, display_y, x_count * this.tile_width, -1 * this.tile_height);
				this.extraClips.push(tileClip);
				x_count++;
			}
		}
		else if (x_offset == 1) {
			var display_x = right;
			var y_count = 0;
			for (var display_y=top; display_y<=bottom; display_y++) {
				var tileClip = this.placeTileClip(display_x, display_y, this.view_width * this.tile_width, y_count * this.tile_height);
				this.extraClips.push(tileClip);
				y_count++;
			}
		}
		else if (x_offset == -1) {
			var display_x = left;
			var y_count = 0;
			for (var display_y=top; display_y<=bottom; display_y++) {
				var tileClip = this.placeTileClip(display_x, display_y, -1 * this.tile_width, y_count * this.tile_height);
				this.extraClips.push(tileClip);
				y_count++;
			}
		}
		
		var new_x = (-1 * x_offset) * this.tile_width;
		var new_y = (-1 * y_offset) * this.tile_height;
		this.tileStub.motor = new Motor(this.tileStub, MapViewer.tileMoveDuration, new_x, new_y);
		this.tileStub.motor.method = "cubeIn";
		this.tileStub.motor.viewer = this;
		this.tileStub.motor.onDone = function () {
			var keyHandler = KeyHandler.getKeyHandler();
			keyHandler.keyPressOkay = true;
			this.viewer.displayView();
		}
		this.tileStub.motor.run();
		
	}
	
	private function removeViewClips()
	{
		//  removes all the tile movieclips from the view
		for (var i in this.tileClips) {
			for (var j in this.tileClips[i]) {
				this.tileClips[i][j].removeMovieClip();
			}
		}
		for (var i in this.extraClips) {
			this.extraClips[i].removeMovieClip();
		}
		this.tileClips = new Array();
		this.extraClips = new Array();
	}
	
	private function checkUnseen (square, tileClip)
	{
		// set unseen state on new tiles
		if (square.seen == false and tileClip.mcUnseen == undefined) {
			var mcUnseen = tileClip.attachMovie("tile_unseen", "mcUnseen", SerialIterator.getSerial());
			mcUnseen._x = 0;
			mcUnseen._y = 0;
			mcUnseen._visible = true;
		}
	}
	
	private function getFrameHash (x, y, frames)
	{
		return(Math.floor( (x+5) * (y+7) * (143+this.tile_seed) /  (x+y+17) ) % frames + 1)
	}
	
	// Event Handlers
	
	private function MoveView (obj)
	{
		var direction = obj.data.direction;
		switch (direction) {
			case "down":
				this.moveView(0,1);
				break;
			case "up":
				this.moveView(0,-1);
				break;
			case "left":
				this.moveView(-1,0);
				break;
			case "right":
				this.moveView(1,0);
				break;
		}
	}
	
	// KeyHandler callback
	
	private function map_viewer_key_handler (target, keyCode) {
		var x = target.model.expedition.x;
		var y = target.model.expedition.y;
		var next_x = x;
		var next_y = y;
		var doMove = false;
		var moveDir = undefined;
		
		switch (keyCode) {
			case Key.DOWN:
				moveDir = "down";
				doMove = true;
				next_y++;
				break;
			case Key.UP:
				moveDir = "up";
				doMove = true;
				next_y--;
				break;
			case Key.LEFT:
				moveDir = "left";
				doMove = true;
				next_x--;
				break;
			case Key.RIGHT:
				doMove = true;
				moveDir = "right";
				next_x++;
				break;
		}
		
		if (doMove) {
			if (target.map.getSquare(x,y).type eq "j" and target.map.getSquare(next_x, next_y).type eq "j") {
				target.broadcaster.broadcast("DisplayAlert", {text: Codex.getAlert("no_jungle")});
			}
			else if (target.map.getSquare(next_x,next_y).type eq "m") {
				target.broadcaster.broadcast("DisplayAlert", {text: Codex.getAlert("no_mountain")});
			}
			else if (target.map.canMove(next_x,next_y)) {
				target.model.expedition.x = next_x;
				target.model.expedition.y = next_y;
				target.broadcaster.broadcast("MoveView", {direction: moveDir});
				target.model.addHours(1);
			}
			else {
				target.broadcaster.broadcast("DisplayAlert", {text: Codex.getAlert("no_go")});
			}
		}
	}


}
