import com.gizmet.util.*;

class com.gizmet.util.KeyHandler extends Object
{
	// KeyHandler is a general-purpose singleton class that is intended to
	// be the primary receiver and distributor of keypresses for an application.
	// Objects can grab an instance of KeyHandler, assign keypress handlers
	// for itself, and tell it when to set focus on what, and how it handles that.
	
	public var onKeyDown:Function;
	
	private static var instanced:Boolean = false;
	private static var key_handler:KeyHandler = undefined;

	private var currentFocus:String;
	private var lastFocus:String;
	private var keyCallbacks:Object;
	private var keyTargets:Object;
	public var keyPressOkay:Boolean = true;
	
	private function KeyHandler () 
	{
		Logger.log("KeyHandler instanced.");
		KeyHandler.instanced = true;
		this.keyCallbacks = new Object();
		this.keyTargets = new Object();

		// set up keypress handler
		this.onKeyDown = function () {doOnKeyDown(Key.getCode())}; 
		Key.addListener(this);
	}
	
	public function doOnKeyDown (keyCode) {
		if (this.keyPressOkay) {
			Logger.log("KeyHandler sending keyCode "+keyCode+" to target: "+this.currentFocus);
			this.keyCallbacks[this.currentFocus](this.keyTargets[this.currentFocus], keyCode);
		}
	}
		
	public static function getKeyHandler ()
	{
		if (!KeyHandler.instanced) {
			KeyHandler.key_handler = new KeyHandler();
		}
		return KeyHandler.key_handler;
	}

	//
	
	public function setFocus (label)
	{
		this.lastFocus = this.currentFocus;
		this.currentFocus = label;
	}

	public function restoreLastFocus () 
	{
		this.currentFocus = this.lastFocus;
	}
	
	public function setCallbacks (label, target, keyFunc)
	{
		this.keyTargets[label] = target;
		this.keyCallbacks[label] = keyFunc;
	}
	
	public function sendKey (label, keyCode)
	{
		Logger.log("KeyHandler passing keyCode "+keyCode+" to target: "+label);
		this.keyCallbacks[label](this.keyTargets[label], keyCode);
	}
	
	
}
